@@ -8,17 +8,17 @@ c aplb1  :   computes     C = A+B  [Sorted version: A, B, C sorted]    c
 c aplsb  :   computes     C = A + s B                                  c
 c diamua :   Computes     C = Diag * A                                 c
 c amudia :   Computes     C = A* Diag                                  c
-c aplsca :   Computes     A:= A + s I    (s = scalar)                  c 
-c----------------------------------------------------------------------c 
+c aplsca :   Computes     A:= A + s I    (s = scalar)                  c
+c----------------------------------------------------------------------c
       subroutine diamua (nrow,job, a, ja, ia, diag, b, jb, ib)
-      real*8 a(*), b(*), diag(nrow), scal
-      integer ja(*),jb(*), ia(nrow+1),ib(nrow+1) 
+      real(kind=8) a(*), b(*), diag(nrow), scal
+      integer ja(*),jb(*), ia(nrow+1),ib(nrow+1)
 c-----------------------------------------------------------------------
-c performs the matrix by matrix product B = Diag * A  (in place) 
+c performs the matrix by matrix product B = Diag * A  (in place)
 c-----------------------------------------------------------------------
 c on entry:
 c ---------
-c nrow	= integer. The row dimension of A
+c nrow  = integer. The row dimension of A
 c
 c job   = integer. job indicator. Job=0 means get array b only
 c         job = 1 means get b, and the integer arrays ib, jb.
@@ -26,46 +26,46 @@ c
 c a,
 c ja,
 c ia   = Matrix A in compressed sparse row format.
-c 
+c
 c diag = diagonal matrix stored as a vector dig(1:n)
 c
 c on return:
 c----------
 c
-c b, 
-c jb, 
-c ib	= resulting matrix B in compressed sparse row sparse format.
-c	    
+c b,
+c jb,
+c ib   = resulting matrix B in compressed sparse row sparse format.
+c
 c Notes:
 c-------
-c 1)        The column dimension of A is not needed. 
+c 1)        The column dimension of A is not needed.
 c 2)        algorithm in place (B can take the place of A).
 c           in this case use job=0.
 c-----------------------------------------------------------------
       do 1 ii=1,nrow
-c     
-c     normalize each row 
-c     
+c
+c     normalize each row
+c
          k1 = ia(ii)
          k2 = ia(ii+1)-1
-         scal = diag(ii) 
+         scal = diag(ii)
          do 2 k=k1, k2
             b(k) = a(k)*scal
  2       continue
  1    continue
-c     
+c
       if (job .eq. 0) return
-c     
+c
       do 3 ii=1, nrow+1
          ib(ii) = ia(ii)
  3    continue
-      do 31 k=ia(1), ia(nrow+1) -1 
+      do 31 k=ia(1), ia(nrow+1) -1
          jb(k) = ja(k)
  31   continue
       return
 c----------end-of-diamua------------------------------------------------
 c-----------------------------------------------------------------------
-      end 
+      end
 c----------------------------------------------------------------------c
 c                          S P A R S K I T                             c
 c----------------------------------------------------------------------c
@@ -99,119 +99,119 @@ c           format (linpack style).                                    c
 c----------------------------------------------------------------------c
       subroutine csrcsc (n,job,ipos,a,ja,ia,ao,jao,iao)
       integer ia(n+1),iao(n+1),ja(*),jao(*)
-      real*8  a(*),ao(*)
+      real(kind=8)  a(*),ao(*)
 c-----------------------------------------------------------------------
 c Compressed Sparse Row     to      Compressed Sparse Column
 c
-c (transposition operation)   Not in place. 
-c----------------------------------------------------------------------- 
+c (transposition operation)   Not in place.
+c-----------------------------------------------------------------------
 c -- not in place --
 c this subroutine transposes a matrix stored in a, ja, ia format.
 c ---------------
 c on entry:
 c----------
-c n	= dimension of A.
-c job	= integer to indicate whether to fill the values (job.eq.1) of the
+c n     = dimension of A.
+c job   = integer to indicate whether to fill the values (job.eq.1) of the
 c         matrix ao or only the pattern., i.e.,ia, and ja (job .ne.1)
 c
 c ipos  = starting position in ao, jao of the transposed matrix.
 c         the iao array takes this into account (thus iao(1) is set to ipos.)
 c         Note: this may be useful if one needs to append the data structure
 c         of the transpose to that of A. In this case use for example
-c                call csrcsc (n,1,ia(n+1),a,ja,ia,a,ja,ia(n+2)) 
-c	  for any other normal usage, enter ipos=1.
-c a	= real array of length nnz (nnz=number of nonzero elements in input 
+c                call csrcsc (n,1,ia(n+1),a,ja,ia,a,ja,ia(n+2))
+c         for any other normal usage, enter ipos=1.
+c a     = real array of length nnz (nnz=number of nonzero elements in input
 c         matrix) containing the nonzero elements.
-c ja	= integer array of length nnz containing the column positions
-c 	  of the corresponding elements in a.
-c ia	= integer of size n+1. ia(k) contains the position in a, ja of
-c	  the beginning of the k-th row.
+c ja    = integer array of length nnz containing the column positions
+c         of the corresponding elements in a.
+c ia    = integer of size n+1. ia(k) contains the position in a, ja of
+c         the beginning of the k-th row.
 c
 c on return:
-c ---------- 
+c ----------
 c output arguments:
-c ao	= real array of size nzz containing the "a" part of the transpose
-c jao	= integer array of size nnz containing the column indices.
-c iao	= integer array of size n+1 containing the "ia" index array of
-c	  the transpose. 
+c ao    = real array of size nzz containing the "a" part of the transpose
+c jao   = integer array of size nnz containing the column indices.
+c iao   = integer array of size n+1 containing the "ia" index array of
+c         the transpose.
 c
-c----------------------------------------------------------------------- 
+c-----------------------------------------------------------------------
       call csrcsc2 (n,n,job,ipos,a,ja,ia,ao,jao,iao)
       end
       subroutine csrcsc2 (n,n2,job,ipos,a,ja,ia,ao,jao,iao)
       integer ia(n+1),iao(n2+1),ja(*),jao(*)
-      real*8  a(*),ao(*)
+      real(kind=8)  a(*),ao(*)
 c-----------------------------------------------------------------------
 c Compressed Sparse Row     to      Compressed Sparse Column
 c
-c (transposition operation)   Not in place. 
-c----------------------------------------------------------------------- 
+c (transposition operation)   Not in place.
+c-----------------------------------------------------------------------
 c Rectangular version.  n is number of rows of CSR matrix,
 c                       n2 (input) is number of columns of CSC matrix.
-c----------------------------------------------------------------------- 
+c-----------------------------------------------------------------------
 c -- not in place --
 c this subroutine transposes a matrix stored in a, ja, ia format.
 c ---------------
 c on entry:
 c----------
-c n	= number of rows of CSR matrix.
+c n     = number of rows of CSR matrix.
 c n2    = number of columns of CSC matrix.
-c job	= integer to indicate whether to fill the values (job.eq.1) of the
+c job   = integer to indicate whether to fill the values (job.eq.1) of the
 c         matrix ao or only the pattern., i.e.,ia, and ja (job .ne.1)
 c
 c ipos  = starting position in ao, jao of the transposed matrix.
 c         the iao array takes this into account (thus iao(1) is set to ipos.)
 c         Note: this may be useful if one needs to append the data structure
 c         of the transpose to that of A. In this case use for example
-c                call csrcsc2 (n,n,1,ia(n+1),a,ja,ia,a,ja,ia(n+2)) 
-c	  for any other normal usage, enter ipos=1.
-c a	= real array of length nnz (nnz=number of nonzero elements in input 
+c                call csrcsc2 (n,n,1,ia(n+1),a,ja,ia,a,ja,ia(n+2))
+c         for any other normal usage, enter ipos=1.
+c a     = real array of length nnz (nnz=number of nonzero elements in input
 c         matrix) containing the nonzero elements.
-c ja	= integer array of length nnz containing the column positions
-c 	  of the corresponding elements in a.
-c ia	= integer of size n+1. ia(k) contains the position in a, ja of
-c	  the beginning of the k-th row.
+c ja    = integer array of length nnz containing the column positions
+c         of the corresponding elements in a.
+c ia    = integer of size n+1. ia(k) contains the position in a, ja of
+c         the beginning of the k-th row.
 c
 c on return:
-c ---------- 
+c ----------
 c output arguments:
-c ao	= real array of size nzz containing the "a" part of the transpose
-c jao	= integer array of size nnz containing the column indices.
-c iao	= integer array of size n+1 containing the "ia" index array of
-c	  the transpose. 
+c ao    = real array of size nzz containing the "a" part of the transpose
+c jao   = integer array of size nnz containing the column indices.
+c iao   = integer array of size n+1 containing the "ia" index array of
+c         the transpose.
 c
-c----------------------------------------------------------------------- 
+c-----------------------------------------------------------------------
 c----------------- compute lengths of rows of transp(A) ----------------
       do 1 i=1,n2+1
          iao(i) = 0
  1    continue
       do 3 i=1, n
-         do 2 k=ia(i), ia(i+1)-1 
+         do 2 k=ia(i), ia(i+1)-1
             j = ja(k)+1
             iao(j) = iao(j)+1
- 2       continue 
+ 2       continue
  3    continue
 c---------- compute pointers from lengths ------------------------------
-      iao(1) = ipos 
+      iao(1) = ipos
       do 4 i=1,n2
          iao(i+1) = iao(i) + iao(i+1)
  4    continue
-c--------------- now do the actual copying ----------------------------- 
+c--------------- now do the actual copying -----------------------------
       do 6 i=1,n
-         do 62 k=ia(i),ia(i+1)-1 
-            j = ja(k) 
+         do 62 k=ia(i),ia(i+1)-1
+            j = ja(k)
             next = iao(j)
             if (job .eq. 1)  ao(next) = a(k)
             jao(next) = i
             iao(j) = next+1
  62      continue
  6    continue
-c-------------------------- reshift iao and leave ---------------------- 
+c-------------------------- reshift iao and leave ----------------------
       do 7 i=n2,1,-1
          iao(i+1) = iao(i)
  7    continue
       iao(1) = ipos
-c--------------- end of csrcsc2 ---------------------------------------- 
+c--------------- end of csrcsc2 ----------------------------------------
 c-----------------------------------------------------------------------
       end
 c----------------------------------------------------------------------c
@@ -232,46 +232,46 @@ c roscal : scales the rows of a matrix by their norms.                 c
 c----------------------------------------------------------------------c
       subroutine rperm (nrow,a,ja,ia,ao,jao,iao,perm,job)
       integer nrow,ja(*),ia(nrow+1),jao(*),iao(nrow+1),perm(nrow),job
-      real*8 a(*),ao(*) 
+      real(kind=8) a(*),ao(*)
 c-----------------------------------------------------------------------
-c this subroutine permutes the rows of a matrix in CSR format. 
-c rperm  computes B = P A  where P is a permutation matrix.  
-c the permutation P is defined through the array perm: for each j, 
-c perm(j) represents the destination row number of row number j. 
+c this subroutine permutes the rows of a matrix in CSR format.
+c rperm  computes B = P A  where P is a permutation matrix.
+c the permutation P is defined through the array perm: for each j,
+c perm(j) represents the destination row number of row number j.
 c Youcef Saad -- recoded Jan 28, 1991.
 c-----------------------------------------------------------------------
 c on entry:
 c----------
-c n 	= dimension of the matrix
+c n      = dimension of the matrix
 c a, ja, ia = input matrix in csr format
-c perm 	= integer array of length nrow containing the permutation arrays
-c	  for the rows: perm(i) is the destination of row i in the
-c         permuted matrix. 
-c         ---> a(i,j) in the original matrix becomes a(perm(i),j) 
+c perm    = integer array of length nrow containing the permutation arrays
+c           for the rows: perm(i) is the destination of row i in the
+c           permuted matrix.
+c         ---> a(i,j) in the original matrix becomes a(perm(i),j)
 c         in the output  matrix.
 c
-c job	= integer indicating the work to be done:
-c 		job = 1	permute a, ja, ia into ao, jao, iao 
+c job     = integer indicating the work to be done:
+c       job = 1   permute a, ja, ia into ao, jao, iao
 c                       (including the copying of real values ao and
 c                       the array iao).
-c 		job .ne. 1 :  ignore real values.
+c       job .ne. 1 :  ignore real values.
 c                     (in which case arrays a and ao are not needed nor
 c                      used).
 c
 c------------
-c on return: 
-c------------ 
+c on return:
+c------------
 c ao, jao, iao = input matrix in a, ja, ia format
-c note : 
+c note :
 c        if (job.ne.1)  then the arrays a and ao are not used.
 c----------------------------------------------------------------------c
 c           Y. Saad, May  2, 1990                                      c
 c----------------------------------------------------------------------c
       logical values
-      values = (job .eq. 1) 
-c     
-c     determine pointers for output matix. 
-c     
+      values = (job .eq. 1)
+c
+c     determine pointers for output matix.
+c
       do 50 j=1,nrow
          i = perm(j)
          iao(i+1) = ia(j+1) - ia(j)
@@ -284,71 +284,71 @@ c
          iao(j+1)=iao(j+1)+iao(j)
  51   continue
 c
-c copying 
+c copying
 c
       do 100 ii=1,nrow
 c
 c old row = ii  -- new row = iperm(ii) -- ko = new pointer
-c        
-         ko = iao(perm(ii)) 
-         do 60 k=ia(ii), ia(ii+1)-1 
-            jao(ko) = ja(k) 
+c
+         ko = iao(perm(ii))
+         do 60 k=ia(ii), ia(ii+1)-1
+            jao(ko) = ja(k)
             if (values) ao(ko) = a(k)
             ko = ko+1
  60      continue
  100  continue
 c
       return
-c---------end-of-rperm ------------------------------------------------- 
+c---------end-of-rperm -------------------------------------------------
 c-----------------------------------------------------------------------
       end
-      subroutine cperm (nrow,a,ja,ia,ao,jao,iao,perm,job) 
+      subroutine cperm (nrow,a,ja,ia,ao,jao,iao,perm,job)
       integer nrow,ja(*),ia(nrow+1),jao(*),iao(nrow+1),perm(*), job
-      real*8 a(*), ao(*) 
+      real(kind=8) a(*), ao(*)
 c-----------------------------------------------------------------------
 c this subroutine permutes the columns of a matrix a, ja, ia.
 c the result is written in the output matrix  ao, jao, iao.
 c cperm computes B = A P, where  P is a permutation matrix
-c that maps column j into column perm(j), i.e., on return 
-c      a(i,j) becomes a(i,perm(j)) in new matrix 
-c Y. Saad, May 2, 1990 / modified Jan. 28, 1991. 
+c that maps column j into column perm(j), i.e., on return
+c      a(i,j) becomes a(i,perm(j)) in new matrix
+c Y. Saad, May 2, 1990 / modified Jan. 28, 1991.
 c-----------------------------------------------------------------------
 c on entry:
 c----------
-c nrow 	= row dimension of the matrix
+c nrow  = row dimension of the matrix
 c
-c a, ja, ia = input matrix in csr format. 
+c a, ja, ia = input matrix in csr format.
 c
-c perm	= integer array of length ncol (number of columns of A
-c         containing the permutation array  the columns: 
+c perm  = integer array of length ncol (number of columns of A
+c         containing the permutation array  the columns:
 c         a(i,j) in the original matrix becomes a(i,perm(j))
 c         in the output matrix.
 c
-c job	= integer indicating the work to be done:
-c 		job = 1	permute a, ja, ia into ao, jao, iao 
+c job   = integer indicating the work to be done:
+c       job = 1   permute a, ja, ia into ao, jao, iao
 c                       (including the copying of real values ao and
 c                       the array iao).
-c 		job .ne. 1 :  ignore real values ao and ignore iao.
+c       job .ne. 1 :  ignore real values ao and ignore iao.
 c
 c------------
-c on return: 
-c------------ 
+c on return:
+c------------
 c ao, jao, iao = input matrix in a, ja, ia format (array ao not needed)
 c
 c Notes:
-c------- 
+c-------
 c 1. if job=1 then ao, iao are not used.
-c 2. This routine is in place: ja, jao can be the same. 
-c 3. If the matrix is initially sorted (by increasing column number) 
-c    then ao,jao,iao  may not be on return. 
-c 
+c 2. This routine is in place: ja, jao can be the same.
+c 3. If the matrix is initially sorted (by increasing column number)
+c    then ao,jao,iao  may not be on return.
+c
 c----------------------------------------------------------------------c
 c local parameters:
       integer k, i, nnz
 c
       nnz = ia(nrow+1)-1
       do 100 k=1,nnz
-         jao(k) = perm(ja(k)) 
+         jao(k) = perm(ja(k))
  100  continue
 c
 c     done with ja array. return if no need to touch values.
@@ -356,7 +356,7 @@ c
       if (job .ne. 1) return
 c
 c else get new pointers -- and copy values too.
-c 
+c
       do 1 i=1, nrow+1
          iao(i) = ia(i)
  1    continue
@@ -366,37 +366,37 @@ c
  2    continue
 c
       return
-c---------end-of-cperm-------------------------------------------------- 
+c---------end-of-cperm--------------------------------------------------
 c-----------------------------------------------------------------------
       end
-      subroutine diapos  (n,ja,ia,idiag) 
-      integer ia(n+1), ja(*), idiag(n) 
+      subroutine diapos  (n,ja,ia,idiag)
+      integer ia(n+1), ja(*), idiag(n)
 c-----------------------------------------------------------------------
 c this subroutine returns the positions of the diagonal elements of a
 c sparse matrix a, ja, ia, in the array idiag.
 c-----------------------------------------------------------------------
 c on entry:
-c---------- 
+c----------
 c
-c n	= integer. row dimension of the matrix a.
+c n     = integer. row dimension of the matrix a.
 c a,ja,
 c    ia = matrix stored compressed sparse row format. a array skipped.
 c
 c on return:
 c-----------
-c idiag  = integer array of length n. The i-th entry of idiag 
+c idiag  = integer array of length n. The i-th entry of idiag
 c          points to the diagonal element a(i,i) in the arrays
 c          a, ja. (i.e., a(idiag(i)) = element A(i,i) of matrix A)
 c          if no diagonal element is found the entry is set to 0.
 c----------------------------------------------------------------------c
 c           Y. Saad, March, 1990
 c----------------------------------------------------------------------c
-      do 1 i=1, n 
+      do 1 i=1, n
          idiag(i) = 0
  1    continue
-c     
-c     sweep through data structure. 
-c     
+c
+c     sweep through data structure.
+c
       do  6 i=1,n
          do 51 k= ia(i),ia(i+1) -1
             if (ja(k) .eq. i) idiag(i) = k
@@ -413,19 +413,19 @@ c does not assume that A is sorted.
 c-----------------------------------------------------------------------
 c on entry:
 c----------
-c n	= integer = the row dimension of the matrix
+c n = integer = the row dimension of the matrix
 c a, ja,
 c    ia = matrix in compressed sparse row format.
-c 
+c
 c on return:
-c----------- 
-c ml	= integer. The bandwidth of the strict lower part of A
-c mu	= integer. The bandwidth of the strict upper part of A 
+c-----------
+c ml   = integer. The bandwidth of the strict lower part of A
+c mu   = integer. The bandwidth of the strict upper part of A
 c
 c Notes:
-c ===== ml and mu are allowed to be negative or return. This may be 
-c       useful since it will tell us whether a band is confined 
-c       in the strict  upper/lower triangular part. 
+c ===== ml and mu are allowed to be negative or return. This may be
+c       useful since it will tell us whether a band is confined
+c       in the strict  upper/lower triangular part.
 c       indeed the definitions of ml and mu are
 c
 c       ml = max ( (i-j)  s.t. a(i,j) .ne. 0  )
@@ -433,42 +433,42 @@ c       mu = max ( (j-i)  s.t. a(i,j) .ne. 0  )
 c----------------------------------------------------------------------c
 c Y. Saad, Sep. 21 1989                                                c
 c----------------------------------------------------------------------c
-      real*8 a(*) 
-      integer ja(*),ia(n+1),ml,mu,ldist,i,k 
+      real(kind=8) a(*)
+      integer ja(*),ia(n+1),ml,mu,ldist,i,k
       ml = - n
       mu = - n
       do 3 i=1,n
-         do 31 k=ia(i),ia(i+1)-1 
+         do 31 k=ia(i),ia(i+1)-1
             ldist = i-ja(k)
             ml = max(ml,ldist)
             mu = max(mu,-ldist)
  31      continue
  3    continue
       return
-c---------------end-of-getbwd ------------------------------------------ 
-c----------------------------------------------------------------------- 
+c---------------end-of-getbwd ------------------------------------------
+c-----------------------------------------------------------------------
       end
-      subroutine infdia (n,ja,ia,ind,idiag) 
+      subroutine infdia (n,ja,ia,ind,idiag)
       integer ia(*), ind(*), ja(*)
 c-----------------------------------------------------------------------
-c     obtains information on the diagonals of A. 
-c----------------------------------------------------------------------- 
+c     obtains information on the diagonals of A.
+c-----------------------------------------------------------------------
 c this subroutine finds the lengths of each of the 2*n-1 diagonals of A
-c it also outputs the number of nonzero diagonals found. 
-c----------------------------------------------------------------------- 
+c it also outputs the number of nonzero diagonals found.
+c-----------------------------------------------------------------------
 c on entry:
-c---------- 
-c n	= dimension of the matrix a.
+c----------
+c n     = dimension of the matrix a.
 c
 c a,    ..... not needed here.
-c ja, 			
+c ja,
 c ia    = matrix stored in csr format
 c
 c on return:
-c----------- 
+c-----------
+c
+c idiag = integer. number of nonzero diagonals found.
 c
-c idiag = integer. number of nonzero diagonals found. 
-c 
 c ind   = integer array of length at least 2*n-1. The k-th entry in
 c         ind contains the number of nonzero elements in the diagonal
 c         number k, the numbering beeing from the lowermost diagonal
@@ -485,10 +485,10 @@ c----------------------------------------------------------------------c
          do 2 k=ia(i),ia(i+1)-1
             j = ja(k)
             ind(n+j-i) = ind(n+j-i) +1
- 2       continue 
+ 2       continue
  3    continue
 c     count the nonzero ones.
-      idiag = 0 
+      idiag = 0
       do 41 k=1, n2
          if (ind(k) .ne. 0) idiag = idiag+1
  41   continue
@@ -497,15 +497,15 @@ c done
 c------end-of-infdia ---------------------------------------------------
 c-----------------------------------------------------------------------
       end
-      subroutine rnrms   (nrow, nrm, a, ja, ia, diag) 
-      real*8 a(*), diag(nrow), scal 
-      integer ja(*), ia(nrow+1) 
+      subroutine rnrms   (nrow, nrm, a, ja, ia, diag)
+      real(kind=8) a(*), diag(nrow), scal
+      integer ja(*), ia(nrow+1)
 c-----------------------------------------------------------------------
 c gets the norms of each row of A. (choice of three norms)
 c-----------------------------------------------------------------------
 c on entry:
 c ---------
-c nrow	= integer. The row dimension of A
+c nrow  = integer. The row dimension of A
 c
 c nrm   = integer. norm indicator. nrm = 1, means 1-norm, nrm =2
 c                  means the 2-nrm, nrm = 0 means max norm
@@ -513,7 +513,7 @@ c
 c a,
 c ja,
 c ia   = Matrix A in compressed sparse row format.
-c 
+c
 c on return:
 c----------
 c
@@ -523,30 +523,30 @@ c-----------------------------------------------------------------
       do 1 ii=1,nrow
 c
 c     compute the norm if each element.
-c     
+c
          scal = 0.0d0
          k1 = ia(ii)
          k2 = ia(ii+1)-1
          if (nrm .eq. 0) then
             do 2 k=k1, k2
-               scal = max(scal,abs(a(k) ) ) 
+               scal = max(scal,abs(a(k) ) )
  2          continue
          elseif (nrm .eq. 1) then
             do 3 k=k1, k2
-               scal = scal + abs(a(k) ) 
+               scal = scal + abs(a(k) )
  3          continue
          else
             do 4 k=k1, k2
                scal = scal+a(k)**2
  4          continue
-         endif 
-         if (nrm .eq. 2) scal = sqrt(scal) 
+         endif
+         if (nrm .eq. 2) scal = sqrt(scal)
          diag(ii) = scal
  1    continue
       return
 c-----------------------------------------------------------------------
 c-------------end-of-rnrms----------------------------------------------
-      end 
+      end
 c----------------------------------------------------------------------c
 c                          S P A R S K I T                             c
 c----------------------------------------------------------------------c
@@ -559,7 +559,7 @@ c QSPLIT  : quick split routine used by ilut to sort out the k largest c
 c           elements in absolute value                                 c
 c----------------------------------------------------------------------c
         subroutine qsplit(a,ind,n,ncut)
-        real*8 a(n)
+        real(kind=8) a(n)
         integer ind(n), n, ncut
 c-----------------------------------------------------------------------
 c     does a quick-sort split of a real array.
@@ -571,7 +571,7 @@ c     abs(a(i)) .le. abs(a(ncut)) for i .gt. ncut
 c
 c     ind(1:n) is an integer array which permuted in the same way as a(*).
 c-----------------------------------------------------------------------
-        real*8 tmp, abskey
+        real(kind=8) tmp, abskey
         integer itmp, first, last
 c-----
         first = 1
@@ -622,9 +622,9 @@ c                          S P A R S K I T                             c
 c----------------------------------------------------------------------c
 c               REORDERING ROUTINES -- LEVEL SET BASED ROUTINES        c
 c----------------------------------------------------------------------c
-c dblstr   : doubled stripe partitioner 
-c BFS      : Breadth-First search traversal algorithm 
-c add_lvst : routine to add a level -- used by BFS 
+c dblstr   : doubled stripe partitioner
+c BFS      : Breadth-First search traversal algorithm
+c add_lvst : routine to add a level -- used by BFS
 c stripes  : finds the level set structure
 c perphn   : finds a pseudo-peripheral node and performs a BFS from it.
 c rversp   : routine to reverse a given permutation (e.g., for RCMK)
@@ -632,202 +632,202 @@ c maskdeg  : integer function to compute the `masked' of a node
 c-----------------------------------------------------------------------
       subroutine BFS(n,ja,ia,nfirst,iperm,mask,maskval,riord,levels,
      *     nlev)
-      implicit none 
+      implicit none
       integer n,ja(*),ia(*),nfirst,iperm(n),mask(n),riord(*),levels(*),
-     *     nlev,maskval 
+     *     nlev,maskval
 c-----------------------------------------------------------------------
 c finds the level-structure (breadth-first-search or CMK) ordering for a
-c given sparse matrix. Uses add_lvst. Allows an set of nodes to be 
-c the initial level (instead of just one node). 
+c given sparse matrix. Uses add_lvst. Allows an set of nodes to be
+c the initial level (instead of just one node).
 c-------------------------parameters------------------------------------
 c on entry:
 c---------
-c     n      = number of nodes in the graph 
+c     n      = number of nodes in the graph
 c     ja, ia = pattern of matrix in CSR format (the ja,ia arrays of csr data
 c     structure)
 c     nfirst = number of nodes in the first level that is input in riord
 c     iperm  = integer array indicating in which order to  traverse the graph
-c     in order to generate all connected components. 
+c     in order to generate all connected components.
 c     if iperm(1) .eq. 0 on entry then BFS will traverse the nodes
 c     in the  order 1,2,...,n.
-c     
-c     riord  = (also an ouput argument). On entry riord contains the labels  
-c     of the nfirst nodes that constitute the first level.      
-c     
-c     mask   = array used to indicate whether or not a node should be 
+c
+c     riord  = (also an ouput argument). On entry riord contains the labels
+c     of the nfirst nodes that constitute the first level.
+c
+c     mask   = array used to indicate whether or not a node should be
 c     condidered in the graph. see maskval.
-c     mask is also used as a marker of  visited nodes. 
-c     
-c     maskval= consider node i only when:  mask(i) .eq. maskval 
-c     maskval must be .gt. 0. 
-c     thus, to consider all nodes, take mask(1:n) = 1. 
-c     maskval=1 (for example) 
-c     
+c     mask is also used as a marker of  visited nodes.
+c
+c     maskval= consider node i only when:  mask(i) .eq. maskval
+c     maskval must be .gt. 0.
+c     thus, to consider all nodes, take mask(1:n) = 1.
+c     maskval=1 (for example)
+c
 c     on return
 c     ---------
-c     mask   = on return mask is restored to its initial state. 
+c     mask   = on return mask is restored to its initial state.
 c     riord  = `reverse permutation array'. Contains the labels of the nodes
 c     constituting all the levels found, from the first level to
-c     the last. 
+c     the last.
 c     levels = pointer array for the level structure. If lev is a level
 c     number, and k1=levels(lev),k2=levels(lev+1)-1, then
 c     all the nodes of level number lev are:
-c     riord(k1),riord(k1+1),...,riord(k2) 
+c     riord(k1),riord(k1+1),...,riord(k2)
 c     nlev   = number of levels found
 c-----------------------------------------------------------------------
-c     
-      integer j, ii, nod, istart, iend 
+c
+      integer j, ii, nod, istart, iend
       logical permut
-      permut = (iperm(1) .ne. 0) 
-c     
-c     start pointer structure to levels 
-c     
-      nlev   = 0 
-c     
+      permut = (iperm(1) .ne. 0)
+c
+c     start pointer structure to levels
+c
+      nlev   = 0
+c
 c     previous end
-c     
-      istart = 0 
+c
+      istart = 0
       ii = 0
-c     
-c     current end 
-c     
+c
+c     current end
+c
       iend = nfirst
-c     
-c     intialize masks to zero -- except nodes of first level -- 
-c     
-      do 12 j=1, nfirst 
-         mask(riord(j)) = 0 
+c
+c     intialize masks to zero -- except nodes of first level --
+c
+      do 12 j=1, nfirst
+         mask(riord(j)) = 0
  12   continue
 c-----------------------------------------------------------------------
- 13   continue 
-c     
+      continue
+c
  1    nlev = nlev+1
       levels(nlev) = istart + 1
-      call add_lvst (istart,iend,nlev,riord,ja,ia,mask,maskval) 
+      call add_lvst (istart,iend,nlev,riord,ja,ia,mask,maskval)
       if (istart .lt. iend) goto 1
- 2    ii = ii+1 
+ 2    ii = ii+1
       if (ii .le. n) then
-         nod = ii         
-         if (permut) nod = iperm(nod)          
+         nod = ii
+         if (permut) nod = iperm(nod)
          if (mask(nod) .eq. maskval) then
-c     
+c
 c     start a new level
 c
             istart = iend
-            iend = iend+1 
+            iend = iend+1
             riord(iend) = nod
             mask(nod) = 0
             goto 1
-         else 
+         else
             goto 2
          endif
       endif
-c----------------------------------------------------------------------- 
- 3    levels(nlev+1) = iend+1 
+c-----------------------------------------------------------------------
+      levels(nlev+1) = iend+1
       do j=1, iend
-         mask(riord(j)) = maskval 
+         mask(riord(j)) = maskval
       enddo
-c----------------------------------------------------------------------- 
+c-----------------------------------------------------------------------
       return
       end
-      subroutine add_lvst(istart,iend,nlev,riord,ja,ia,mask,maskval) 
-      integer nlev, nod, riord(*), ja(*), ia(*), mask(*) 
+      subroutine add_lvst(istart,iend,nlev,riord,ja,ia,mask,maskval)
+      integer nlev, nod, riord(*), ja(*), ia(*), mask(*)
 c-------------------------------------------------------------
-c     adds one level set to the previous sets.. 
+c     adds one level set to the previous sets..
 c     span all nodes of previous mask
 c-------------------------------------------------------------
       nod = iend
-      do 25 ir = istart+1,iend 
-         i = riord(ir)		
+      do 25 ir = istart+1,iend
+         i = riord(ir)
          do 24 k=ia(i),ia(i+1)-1
             j = ja(k)
             if (mask(j) .eq. maskval) then
-               nod = nod+1 
+               nod = nod+1
                mask(j) = 0
                riord(nod) = j
-            endif 
+            endif
  24      continue
  25   continue
-      istart = iend 
-      iend   = nod 
+      istart = iend
+      iend   = nod
       return
-      end 
+      end
       subroutine stripes (nlev,riord,levels,ip,map,mapptr,ndom)
       implicit none
       integer nlev,riord(*),levels(nlev+1),ip,map(*),
      *    mapptr(*), ndom
 c-----------------------------------------------------------------------
-c    this is a post processor to BFS. stripes uses the output of BFS to 
-c    find a decomposition of the adjacency graph by stripes. It fills 
-c    the stripes level by level until a number of nodes .gt. ip is 
-c    is reached. 
+c    this is a post processor to BFS. stripes uses the output of BFS to
+c    find a decomposition of the adjacency graph by stripes. It fills
+c    the stripes level by level until a number of nodes .gt. ip is
+c    is reached.
 c---------------------------parameters-----------------------------------
-c on entry: 
+c on entry:
 c --------
-c nlev   = number of levels as found by BFS 
-c riord  = reverse permutation array produced by BFS -- 
-c levels = pointer array for the level structure as computed by BFS. If 
-c          lev is a level number, and k1=levels(lev),k2=levels(lev+1)-1, 
+c nlev   = number of levels as found by BFS
+c riord  = reverse permutation array produced by BFS --
+c levels = pointer array for the level structure as computed by BFS. If
+c          lev is a level number, and k1=levels(lev),k2=levels(lev+1)-1,
 c          then all the nodes of level number lev are:
-c                      riord(k1),riord(k1+1),...,riord(k2) 
+c                      riord(k1),riord(k1+1),...,riord(k2)
 c  ip    = number of desired partitions (subdomains) of about equal size.
-c 
+c
 c on return
 c ---------
-c ndom     = number of subgraphs (subdomains) found 
+c ndom     = number of subgraphs (subdomains) found
 c map      = node per processor list. The nodes are listed contiguously
-c            from proc 1 to nproc = mpx*mpy. 
-c mapptr   = pointer array for array map. list for proc. i starts at 
+c            from proc 1 to nproc = mpx*mpy.
+c mapptr   = pointer array for array map. list for proc. i starts at
 c            mapptr(i) and ends at mapptr(i+1)-1 in array map.
 c-----------------------------------------------------------------------
-c local variables. 
+c local variables.
 c
-      integer ib,ktr,ilev,k,nsiz,psiz 
-      ndom = 1 
+      integer ib,ktr,ilev,k,nsiz,psiz
+      ndom = 1
       ib = 1
 c to add: if (ip .le. 1) then ...
-      nsiz = levels(nlev+1) - levels(1) 
-      psiz = (nsiz-ib)/max(1,(ip - ndom + 1)) + 1 
-      mapptr(ndom) = ib 
-      ktr = 0 
+      nsiz = levels(nlev+1) - levels(1)
+      psiz = (nsiz-ib)/max(1,(ip - ndom + 1)) + 1
+      mapptr(ndom) = ib
+      ktr = 0
       do 10 ilev = 1, nlev
 c
 c     add all nodes of this level to domain
-c     
+c
          do 3 k=levels(ilev), levels(ilev+1)-1
             map(ib) = riord(k)
             ib = ib+1
-            ktr = ktr + 1 
-            if (ktr .ge. psiz  .or. k .ge. nsiz) then 
+            ktr = ktr + 1
+            if (ktr .ge. psiz  .or. k .ge. nsiz) then
                ndom = ndom + 1
-               mapptr(ndom) = ib 
-               psiz = (nsiz-ib)/max(1,(ip - ndom + 1)) + 1 
+               mapptr(ndom) = ib
+               psiz = (nsiz-ib)/max(1,(ip - ndom + 1)) + 1
                ktr = 0
             endif
 c
  3       continue
  10   continue
       ndom = ndom-1
-      return 
+      return
       end
-      integer function maskdeg (ja,ia,nod,mask,maskval) 
-      implicit none 
+      integer function maskdeg (ja,ia,nod,mask,maskval)
+      implicit none
       integer ja(*),ia(*),nod,mask(*),maskval
 c-----------------------------------------------------------------------
-      integer deg, k 
-      deg = 0 
+      integer deg, k
+      deg = 0
       do k =ia(nod),ia(nod+1)-1
-         if (mask(ja(k)) .eq. maskval) deg = deg+1 
+         if (mask(ja(k)) .eq. maskval) deg = deg+1
       enddo
-      maskdeg = deg 
+      maskdeg = deg
       return
-      end 
-      subroutine perphn(n,ja,ia,init,mask,maskval,nlev,riord,levels) 
+      end
+      subroutine perphn(n,ja,ia,init,mask,maskval,nlev,riord,levels)
       implicit none
       integer n,ja(*),ia(*),init,mask(*),maskval,
      *     nlev,riord(*),levels(*)
 c-----------------------------------------------------------------------
-c     finds a peripheral node and does a BFS search from it. 
+c     finds a peripheral node and does a BFS search from it.
 c-----------------------------------------------------------------------
 c     see routine  dblstr for description of parameters
 c input:
@@ -845,28 +845,28 @@ c init    = actual pseudo-peripherial node found.
 c nlev    = number of levels in the final BFS traversal.
 c riord   =
 c levels  =
-c----------------------------------------------------------------------- 
+c-----------------------------------------------------------------------
       integer j,nlevp,deg,nfirst,mindeg,nod,maskdeg
-      integer iperm(1) 
-      nlevp = 0 
+      integer iperm(1)
+      nlevp = 0
  1    continue
       riord(1) = init
-      nfirst = 1 
+      nfirst = 1
       iperm(1) = 0
 c
       call BFS(n,ja,ia,nfirst,iperm,mask,maskval,riord,levels,nlev)
-      if (nlev .gt. nlevp) then 
-         mindeg = n+1 
+      if (nlev .gt. nlevp) then
+         mindeg = n+1
          do j=levels(nlev),levels(nlev+1)-1
-            nod = riord(j) 
+            nod = riord(j)
             deg = maskdeg(ja,ia,nod,mask,maskval)
             if (deg .lt. mindeg) then
                init = nod
                mindeg = deg
-            endif 
+            endif
          enddo
-         nlevp = nlev 
-         goto 1 
+         nlevp = nlev
+         goto 1
       endif
       return
       end
